package net.knarcraft.minstrel.listener;

import net.citizensnpcs.api.event.NPCRemoveEvent;
import net.citizensnpcs.api.npc.NPC;
import net.knarcraft.minstrel.MinstrelPlugin;
import net.knarcraft.minstrel.manager.QueueManager;
import net.knarcraft.minstrel.music.SongEndTime;
import net.knarcraft.minstrel.trait.MinstrelTrait;
import net.knarcraft.minstrel.util.SoundHelper;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerChangedWorldEvent;
import org.bukkit.event.player.PlayerMoveEvent;
import org.jetbrains.annotations.NotNull;

/**
 * A listener for minstrel-relate events
 */
public class MinstrelListener implements Listener {

    @EventHandler
    public void enterWorldListener(@NotNull PlayerChangedWorldEvent event) {
        QueueManager.clearQueue(event.getPlayer());
    }

    @EventHandler
    public void playerMoveListener(@NotNull PlayerMoveEvent event) {
        if (event.getTo() == null || event.getFrom().getBlock().equals(event.getTo().getBlock())) {
            return;
        }

        // Start playback for any Minstrels that just entered the player's hearable range
        for (MinstrelTrait minstrelTrait : MinstrelPlugin.getInstance().getMinstrels()) {
            if (!SoundHelper.canHear(event.getPlayer(), minstrelTrait) ||
                    QueueManager.isQueued(event.getPlayer(), minstrelTrait)) {
                continue;
            }

            QueueManager.queue(new SongEndTime(event.getPlayer(), minstrelTrait, 0));
        }
    }

    /**
     * Listens for the deletion of any NPCs, and stops their music
     *
     * @param event <p>The triggered event</p>
     */
    @EventHandler
    public void npcDeletionListener(@NotNull NPCRemoveEvent event) {
        NPC npc = event.getNPC();
        if (npc.hasTrait(MinstrelTrait.class)) {
            MinstrelTrait minstrelTrait = npc.getTraitNullable(MinstrelTrait.class);
            minstrelTrait.getPlaylist().stop();
        }
    }

}
